@@ -990,6 +990,28 @@ on_auto_reorder_button_toggled(GtkToggleButton *button, gpointer data)
     nabi_log(4, "preference: set auto reorder: %d\n", flag);
 }
 
+static void
+on_ignore_app_fontset_button_toggled(GtkToggleButton *button, gpointer data)
+{
+    gboolean flag = gtk_toggle_button_get_active(button);
+
+    config->ignore_app_fontset = flag;
+    nabi_server_set_ignore_app_fontset(nabi_server, flag);
+
+    nabi_log(4, "preference: set ignore app fontset: %d\n", flag);
+}
+
+static void
+on_use_system_keymap_button_toggled(GtkToggleButton *button, gpointer data)
+{
+    gboolean flag = gtk_toggle_button_get_active(button);
+
+    config->use_system_keymap = flag;
+    nabi_server_set_use_system_keymap(nabi_server, flag);
+
+    nabi_log(4, "preference: set use system keymap: %d\n", flag);
+}
+
 static void
 on_default_input_mode_button_toggled(GtkToggleButton *button, gpointer data)
 {
@@ -1133,6 +1155,28 @@ create_advanced_page(GtkWidget* dialog)
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
+    button = gtk_check_button_new_with_label(_("Ignore fontset information from the client"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+			         config->ignore_app_fontset);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-ignore-app-fontset", button);
+    g_signal_connect(G_OBJECT(button), "toggled",
+		     G_CALLBACK(on_ignore_app_fontset_button_toggled), NULL);
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+    button = gtk_check_button_new_with_label(_("Use system keymap"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+			         config->use_system_keymap);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(dialog), "nabi-pref-use-system-keymap", button);
+    g_signal_connect(G_OBJECT(button), "toggled",
+		     G_CALLBACK(on_use_system_keymap_button_toggled), NULL);
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
     label = gtk_label_new(_("Input mode scope: "));
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
@@ -1195,7 +1239,7 @@ on_preference_reset(GtkWidget *button, gpointer data)
     p = g_object_get_data(dialog, "nabi-pref-theme");
     if (p != NULL) {
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(p));
-	path = search_text_in_model(model, THEMES_LIST_NAME, "Jini");
+	path = search_text_in_model(model, THEMES_LIST_NAME, DEFAULT_THEME);
 	if (path != NULL) {
 	    gtk_tree_view_set_cursor (GTK_TREE_VIEW(p), path, NULL, FALSE);
 	    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(p),
@@ -1203,7 +1247,7 @@ on_preference_reset(GtkWidget *button, gpointer data)
 	    gtk_tree_path_free(path);
 	}
     }
-    nabi_app_set_theme("Jini");
+    nabi_app_set_theme(DEFAULT_THEME);
 
     p = g_object_get_data(dialog, "nabi-pref-use-tray-icon");
     if (p != NULL) {
@@ -1314,6 +1358,16 @@ on_preference_reset(GtkWidget *button, gpointer data)
 	gtk_font_button_set_font_name(GTK_FONT_BUTTON(p), "Sans 9");
 	g_signal_emit_by_name(p, "font-set", NULL);
     }
+
+    p = g_object_get_data(dialog, "nabi-pref-ignore-app-fonset");
+    if (p != NULL) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p), FALSE);
+    }
+
+    p = g_object_get_data(dialog, "nabi-pref-use-system-keymap");
+    if (p != NULL) {
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(p), FALSE);
+    }
 }
 
 static void
