@@ -48,16 +48,6 @@ struct KeySymPair {
     KeySym value;
 };
 
-static NabiHangulKeyboard hangul_keyboard_list[] = {
-    { "2",  N_("2 set") },
-    { "32", N_("3 set with 2 set layout") },
-    { "3f", N_("3 set final") },
-    { "39", N_("3 set 390") },
-    { "3s", N_("3 set no-shift") },
-    { "3y", N_("3 set yetguel") },
-    { "ro", N_("Romaja") },
-    { NULL, NULL }
-};
 
 /* from handler.c */
 Bool nabi_handler(XIMS ims, IMProtocol *call_data);
@@ -292,6 +282,8 @@ static char *nabi_locales[] = {
 NabiServer*
 nabi_server_new(Display* display, int screen, const char *name)
 {
+    unsigned i;
+    unsigned n;
     const char *charset;
     char *trigger_keys[3]   = { "Hangul", "Shift+space", NULL };
     char *off_keys[2]       = { "Escape", NULL };
@@ -333,7 +325,6 @@ nabi_server_new(Display* display, int screen, const char *name)
 	/* We add current locale to nabi support  locales array,
 	 * so whatever the locale string is, if its encoding is utf8,
 	 * nabi support the locale */
-	int i;
 	for (i = 0; server->locales[i] != NULL; i++)
 	    continue;
 	server->locales[i] = setlocale(LC_CTYPE, NULL);
@@ -349,7 +340,18 @@ nabi_server_new(Display* display, int screen, const char *name)
     server->layouts = NULL;
     server->layout = NULL;
     server->hangul_keyboard = NULL;
-    server->hangul_keyboard_list = hangul_keyboard_list;
+
+    /* init keyboard list from libhangul */
+    n = hangul_ic_get_n_keyboards();
+    server->hangul_keyboard_list = g_new(NabiHangulKeyboard, n + 1);
+    for (i = 0; i < n; ++i) {
+	const char* id = hangul_ic_get_keyboard_id(i);
+	const char* name = hangul_ic_get_keyboard_name(i);
+	server->hangul_keyboard_list[i].id = id;
+	server->hangul_keyboard_list[i].name= name;
+    }
+    server->hangul_keyboard_list[i].id = NULL;
+    server->hangul_keyboard_list[i].name= NULL;
 
     server->dynamic_event_flow = True;
     server->commit_by_word = False;
@@ -369,6 +371,8 @@ nabi_server_new(Display* display, int screen, const char *name)
     /* options */
     server->show_status = False;
     server->use_simplified_chinese = False;
+    server->ignore_app_fontset = False;
+    server->use_system_keymap = False;
     server->preedit_fg.pixel = 0;
     server->preedit_fg.red = 0xffff;
     server->preedit_fg.green = 0;
@@ -437,6 +441,9 @@ nabi_server_destroy(NabiServer *server)
     if (server->symbol_table != NULL)
 	hanja_table_delete(server->symbol_table);
 
+    /* libhangul keyboard list */
+    g_free(server->hangul_keyboard_list);
+
     g_free(server->trigger_keys.keylist);
     g_free(server->candidate_keys.keylist);
     g_free(server->off_keys.keylist);
@@ -464,14 +471,17 @@ void
 nabi_server_set_mode_info(NabiServer *server, int state)
 {
     long data;
+    Window root;
+    Atom property;
+    Atom type;
 
     if (server == NULL)
 	return;
 
     data = state;
-    Window root = RootWindow(server->display, server->screen);
-    Atom property = XInternAtom(server->display, "_HANGUL_INPUT_MODE", FALSE);
-    Atom type = XInternAtom(server->display, "INTEGER", FALSE);
+    root = RootWindow(server->display, server->screen);
+    property = XInternAtom(server->display, "_HANGUL_INPUT_MODE", FALSE);
+    type = XInternAtom(server->display, "INTEGER", FALSE);
 
     XChangeProperty(server->display, root, property, type, 
 		    32, PropModeReplace, (unsigned char*)&data, 1);
@@ -1060,6 +1070,104 @@ nabi_server_get_current_keyboard_name(NabiServer* server)
     return nabi_server_get_keyboard_name_by_id(server, server->hangul_keyboard);
 }
 
+/* 이 테이블은 US layout을 기준으로 만든것이다.
+ * 그러나 keycode의 값은 하드웨어마다 다를 수 있으므로 
+ * 일반 PC 환경이 아닌 곳에서는 문제가 될지도 모른다. */
+static const unsigned int keymap[][2] = {
+    { XK_1,             XK_exclam         },  /* 10 */
+    { XK_2,             XK_at             },  /* 11 */
+    { XK_3,             XK_numbersign     },  /* 12 */
+    { XK_4,             XK_dollar         },  /* 13 */
+    { XK_5,             XK_percent        },  /* 14 */
+    { XK_6,             XK_asciicircum    },  /* 15 */
+    { XK_7,             XK_ampersand      },  /* 16 */
+    { XK_8,             XK_asterisk       },  /* 17 */
+    { XK_9,             XK_parenleft      },  /* 18 */
+    { XK_0,             XK_parenright     },  /* 19 */
+    { XK_minus,         XK_underscore     },  /* 20 */
+    { XK_equal,         XK_plus           },  /* 21 */
+    { XK_BackSpace,     XK_BackSpace      },  /* 22 */
+    { XK_Tab,           XK_Tab            },  /* 23 */
+    { XK_q,             XK_Q              },  /* 24 */
+    { XK_w,             XK_W              },  /* 25 */
+    { XK_e,             XK_E              },  /* 26 */
+    { XK_r,             XK_R              },  /* 27 */
+    { XK_t,             XK_T              },  /* 28 */
+    { XK_y,             XK_Y              },  /* 29 */
+    { XK_u,             XK_U              },  /* 30 */
+    { XK_i,             XK_I              },  /* 31 */
+    { XK_o,             XK_O              },  /* 32 */
+    { XK_p,             XK_P              },  /* 33 */
+    { XK_bracketleft,   XK_braceleft      },  /* 34 */
+    { XK_bracketright,  XK_braceright     },  /* 35 */
+    { XK_Return,        XK_Return         },  /* 36 */
+    { XK_Control_L,     XK_Control_L      },  /* 37 */
+    { XK_a,             XK_A              },  /* 38 */
+    { XK_s,             XK_S              },  /* 39 */
+    { XK_d,             XK_D              },  /* 40 */
+    { XK_f,             XK_F              },  /* 41 */
+    { XK_g,             XK_G              },  /* 42 */
+    { XK_h,             XK_H              },  /* 43 */
+    { XK_j,             XK_J              },  /* 44 */
+    { XK_k,             XK_K              },  /* 45 */
+    { XK_l,             XK_L              },  /* 46 */
+    { XK_semicolon,     XK_colon          },  /* 47 */
+    { XK_apostrophe,    XK_quotedbl       },  /* 48 */
+    { XK_grave,         XK_asciitilde     },  /* 49 */
+    { XK_Shift_L,       XK_Shift_L        },  /* 50 */
+    { XK_backslash,     XK_bar            },  /* 51 */
+    { XK_z,             XK_Z              },  /* 52 */
+    { XK_x,             XK_X              },  /* 53 */
+    { XK_c,             XK_C              },  /* 54 */
+    { XK_v,             XK_V              },  /* 55 */
+    { XK_b,             XK_B              },  /* 56 */
+    { XK_n,             XK_N              },  /* 57 */
+    { XK_m,             XK_M              },  /* 58 */
+    { XK_comma,         XK_less           },  /* 59 */
+    { XK_period,        XK_greater        },  /* 60 */
+    { XK_slash,         XK_question       },  /* 61 */
+};
+
+KeySym
+nabi_server_lookup_keysym(NabiServer* server, XKeyEvent* event)
+{
+    int index;
+    KeySym keysym;
+
+    keysym = NoSymbol;
+    index = (event->state & ShiftMask) ? 1 : 0;
+
+    /* 자판 설정에 따른 변환 문제를 피하기 위해서 내장 keymap을 사용하여
+     * keycode를 keysym으로 변환함 */
+    if (!server->use_system_keymap) {
+	if (event->keycode >= 10 && event->keycode < 61)
+	    keysym = keymap[event->keycode - 10][index];
+    }
+
+    /* XLookupString()을 사용하지 않고 XLookupKeysym()함수를
+     * 사용한 것은 데스크탑에서 여러 언어 자판을 지원하기위해서 Xkb를
+     * 사용하는 경우에 쉽게 처리하기 위한 방편이다.
+     * Xkb를 사용하게 되면 keymap이 재정의되므로 XLookupString()의 리턴값은
+     * 재정의된 키값을 얻게되어 각 언어(예를 들어 프랑스, 러시아 등)의
+     * 자판에서 일반 qwerty 자판으로 변환을 해줘야 한다. 이 문제를 좀더 
+     * 손쉽게 풀기 위해서 재정의된 자판이 아닌 첫번째 자판의 값을 직접
+     * 가져오기 위해서 XLookupKeysym()함수를 사용한다. */
+    if (keysym == NoSymbol) {
+	keysym = XLookupKeysym(event, index);
+    }
+
+    /* 그러나 이 함수를 사용하게되면 새로 정의된 키를 가져와야 되는 경우에
+     * 못가져오는 수가 생긴다. 이를 피하기 위해서 XLookupKeysym() 함수가
+     * 0을 리턴하면 XLookupString()으로 다시한번 시도하는 방식으로 
+     * 처리한다. */
+    if (keysym == NoSymbol) {
+	char buf[64];
+	XLookupString(event, buf, sizeof(buf), &keysym, NULL);
+    }
+
+    return keysym;
+}
+
 KeySym
 nabi_server_normalize_keysym(NabiServer *server,
 			     KeySym keysym, unsigned int state)
@@ -1164,6 +1272,20 @@ nabi_server_set_simplified_chinese(NabiServer* server, Bool state)
 	server->use_simplified_chinese = state;
 }
 
+void
+nabi_server_set_ignore_app_fontset(NabiServer* server, Bool state)
+{
+    if (server != NULL)
+	server->ignore_app_fontset = state;
+}
+
+void
+nabi_server_set_use_system_keymap(NabiServer* server, Bool state)
+{
+    if (server != NULL)
+	server->use_system_keymap = state;
+}
+
 void
 nabi_server_write_log(NabiServer *server)
 {
