@@ -24,7 +24,6 @@
 #endif
 
 #include <X11/Xlib.h>
-#include <wchar.h>
 #include <time.h>
 
 #include <glib.h>
@@ -104,7 +103,7 @@ struct _NabiServer {
 
     /* hangul automata */
     char*                   hangul_keyboard;
-    const NabiHangulKeyboard* hangul_keyboard_list;
+    NabiHangulKeyboard*     hangul_keyboard_list;
 
     NabiOutputMode          output_mode;
 
@@ -121,6 +120,8 @@ struct _NabiServer {
     Bool                    show_status;
     Bool                    hanja_mode;
     Bool                    use_simplified_chinese;
+    Bool                    ignore_app_fontset;
+    Bool                    use_system_keymap;
     NabiInputMode           default_input_mode;
     NabiInputMode           input_mode;
     NabiInputModeScope      input_mode_scope;
@@ -183,6 +184,8 @@ void        nabi_server_set_default_input_mode(NabiServer* server,
 void        nabi_server_set_input_mode_scope(NabiServer* server,
 					     NabiInputModeScope scope);
 void        nabi_server_set_simplified_chinese(NabiServer* server, Bool state);
+void        nabi_server_set_ignore_app_fontset(NabiServer* server, Bool state);
+void        nabi_server_set_use_system_keymap(NabiServer* server, Bool state);
 
 NabiIC*     nabi_server_get_ic          (NabiServer *server,
 					 CARD16 connect_id, CARD16 icid);
@@ -203,6 +206,7 @@ void            nabi_server_remove_toplevel(NabiServer* server,
 Bool        nabi_server_is_locale_supported(NabiServer *server,
 					    const char *locale);
 Bool        nabi_server_is_valid_str    (NabiServer *server, const char* str);
+KeySym      nabi_server_lookup_keysym(NabiServer* server, XKeyEvent* event);
 KeySym      nabi_server_normalize_keysym(NabiServer *server,
 					 KeySym keysym, unsigned int state);
 void        nabi_server_log_key         (NabiServer *server,
