@@ -31,8 +31,15 @@
 #include <X11/keysym.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
 
+#if GTK_CHECK_VERSION(2, 16, 0)
+#define HAVE_GTK_STATUS_ICON 1
+#endif
+
+#if !defined(HAVE_GTK_STATUS_ICON)
 #include "eggtrayicon.h"
+#endif
 
 #include "debug.h"
 #include "gettext.h"
@@ -68,17 +75,23 @@ typedef struct _NabiPalette {
     guint          source_id;
 } NabiPalette;
 
+#if defined(HAVE_GTK_STATUS_ICON)
+typedef struct _NabiTrayIcon {
+    GtkStatusIcon* icon;
+} NabiTrayIcon;
+#else
 typedef struct _NabiTrayIcon {
     EggTrayIcon*   widget;
     NabiStateIcon* state;
+    GtkTooltips*   tooltips;
 } NabiTrayIcon;
+#endif
 
 static GtkWidget *about_dialog = NULL;
 static GtkWidget *preference_dialog = NULL;
 static GtkWidget *hide_palette_menuitem = NULL;
 
 static void nabi_app_load_base_icons();
-static void nabi_app_update_tooltips();
 
 static void nabi_state_icon_load(NabiStateIcon* state, int w, int h);
 
@@ -89,7 +102,10 @@ static void nabi_palette_destroy(NabiPalette* palette);
 static void nabi_palette_update_hanja_mode(NabiPalette* palette, gboolean state);
 
 static gboolean nabi_tray_icon_create(gpointer data);
+#if !defined(HAVE_GTK_STATUS_ICON)
 static void nabi_tray_load_icons(NabiTrayIcon* tray, gint default_size);
+#endif
+static void nabi_tray_icon_update_tooltips();
 static void nabi_tray_icon_destroy(NabiTrayIcon* tray);
 
 static void remove_event_filter();
@@ -372,6 +388,8 @@ nabi_app_setup_server(void)
     }
 
     nabi_server_set_preedit_font(nabi_server, nabi->config->preedit_font->str);
+    nabi_server_set_ignore_app_fontset(nabi_server,
+				      nabi->config->ignore_app_fontset);
 }
 
 void
@@ -513,10 +531,22 @@ on_tray_icon_embedded(GtkWidget *widget, gpointer data)
     }
 }
 
+#if defined(HAVE_GTK_STATUS_ICON)
+static void
+on_status_icon_embedded(GObject* gobject, GParamSpec* paramspec,
+			     gpointer data)
+{
+    on_tray_icon_embedded(NULL, data);
+}
+#endif
+
 static void
 on_tray_icon_destroyed(GtkWidget *widget, gpointer data)
 {
+#if !defined(HAVE_GTK_STATUS_ICON)
     nabi_state_icon_destroy(nabi_tray->state);
+#endif
+
     g_free(nabi_tray);
     nabi_tray = NULL;
     nabi_log(1, "tray icon is destroyed\n");
@@ -575,6 +605,29 @@ nabi_menu_position_func(GtkMenu *menu,
 	*x = screen_width - menu_width;
 }
 
+#if defined(HAVE_GTK_STATUS_ICON)
+static void
+on_tray_icon_popup_menu(GtkStatusIcon *status_icon,
+			guint button,
+			guint activate_time,
+			gpointer data)
+{
+    static GtkWidget *menu = NULL;
+
+    if (preference_dialog != NULL) {
+	gtk_window_present(GTK_WINDOW(preference_dialog));
+	return;
+    }
+
+    if (menu != NULL)
+	gtk_widget_destroy(menu);
+
+    menu = create_tray_icon_menu();
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
+		   gtk_status_icon_position_menu, status_icon,
+		   button, activate_time);
+}
+#else
 static gboolean
 on_tray_icon_button_press(GtkWidget *widget,
 			  GdkEventButton *event,
@@ -599,7 +652,9 @@ on_tray_icon_button_press(GtkWidget *widget,
 
     return FALSE;
 }
+#endif
 
+#if !defined(HAVE_GTK_STATUS_ICON)
 static void
 on_tray_icon_size_allocate (GtkWidget *widget,
 			    GtkAllocation *allocation,
@@ -631,6 +686,7 @@ on_tray_icon_size_allocate (GtkWidget *widget,
 	nabi_tray_load_icons(nabi_tray, size);
     }
 }
+#endif /* !defined(HAVE_GTK_STATUS_ICON) */
 
 static void get_statistic_string(GString *str)
 {
@@ -775,7 +831,7 @@ on_menu_about(GtkWidget *widget)
     comment_str = g_strdup_printf(
 		    _("<span size=\"large\">An Easy Hangul XIM</span>\n"
 		      "version %s\n\n"
-		      "Copyright (C) 2003-2010 Choe Hwanjin"),
+		      "Copyright (C) 2003-2011 Choe Hwanjin"),
 		    VERSION);
     gtk_label_set_markup(GTK_LABEL(comment), comment_str);
     gtk_label_set_justify(GTK_LABEL(comment), GTK_JUSTIFY_CENTER);
@@ -1003,6 +1059,7 @@ create_tray_icon_menu(void)
     return menu;
 }
 
+#if !defined(HAVE_GTK_STATUS_ICON)
 static void
 nabi_tray_load_icons(NabiTrayIcon* tray, gint default_size)
 {
@@ -1020,12 +1077,29 @@ nabi_tray_load_icons(NabiTrayIcon* tray, gint default_size)
 
     nabi_state_icon_load(tray->state, w, h);
 }
+#endif
 
 static void
 nabi_tray_update_state(NabiTrayIcon* tray, int state)
 {
+#if defined(HAVE_GTK_STATUS_ICON)
+    if (tray != NULL) {
+	switch (state) {
+	case 1:
+	    gtk_status_icon_set_from_pixbuf(nabi_tray->icon, english_pixbuf);
+	    break;
+	case 2:
+	    gtk_status_icon_set_from_pixbuf(nabi_tray->icon, hangul_pixbuf);
+	    break;
+	default:
+	    gtk_status_icon_set_from_pixbuf(nabi_tray->icon, none_pixbuf);
+	    break;
+	}
+    }
+#else
     if (tray != NULL)
 	nabi_state_icon_update(tray->state, state);
+#endif
 }
 
 static GdkFilterReturn
@@ -1164,13 +1238,32 @@ nabi_app_load_base_icons()
 	gtk_dialog_run(GTK_DIALOG(message));
 	gtk_widget_destroy(message);
 
-	g_string_assign(nabi->config->theme, "Jini");
+	g_string_assign(nabi->config->theme, DEFAULT_THEME);
     }
 }
 
 static gboolean
 nabi_tray_icon_create(gpointer data)
 {
+#if defined(HAVE_GTK_STATUS_ICON)
+    NabiTrayIcon* tray;
+
+    if (nabi_tray != NULL)
+	return FALSE;
+
+    tray = g_new(NabiTrayIcon, 1);
+    nabi_tray = tray;
+
+    tray->icon = gtk_status_icon_new();
+    g_signal_connect(G_OBJECT(tray->icon), "popup-menu",
+		     G_CALLBACK(on_tray_icon_popup_menu), NULL);
+
+    gtk_status_icon_set_from_pixbuf(tray->icon, none_pixbuf);
+    nabi_tray_icon_update_tooltips();
+
+    g_signal_connect(G_OBJECT(tray->icon), "notify::embedded",
+		     G_CALLBACK(on_status_icon_embedded), tray);
+#else
     NabiTrayIcon* tray;
     GtkWidget *eventbox;
     GtkTooltips *tooltips;
@@ -1195,8 +1288,8 @@ nabi_tray_icon_create(gpointer data)
     nabi->tray_icon = eventbox;
 
     tooltips = gtk_tooltips_new();
-    nabi->tooltips = tooltips;
-    nabi_app_update_tooltips();
+    tray->tooltips = tooltips;
+    nabi_tray_icon_update_tooltips();
 
     orientation = egg_tray_icon_get_orientation(tray->widget);
     if (orientation == GTK_ORIENTATION_VERTICAL) {
@@ -1218,6 +1311,7 @@ nabi_tray_icon_create(gpointer data)
     g_signal_connect(G_OBJECT(tray->widget), "destroy",
 		     G_CALLBACK(on_tray_icon_destroyed), tray);
     gtk_widget_show(GTK_WIDGET(tray->widget));
+#endif // defined(HAVE_GTK_STATUS_ICON)
 
     return FALSE;
 }
@@ -1225,9 +1319,16 @@ nabi_tray_icon_create(gpointer data)
 static void
 nabi_tray_icon_destroy(NabiTrayIcon* tray)
 {
+#if defined(HAVE_GTK_STATUS_ICON)
+    if (tray->icon != NULL) {
+	g_object_unref(tray->icon);
+	on_tray_icon_destroyed(NULL, NULL);
+    }
+#else
     // widget만 destroy하면 destroy signal에서 나머지 delete 처리를 한다.
     if (tray->widget != NULL)
 	gtk_widget_destroy(GTK_WIDGET(tray->widget));
+#endif
 }
 
 static void
@@ -1438,8 +1539,17 @@ nabi_app_set_theme(const gchar *name)
     g_string_assign(nabi->config->theme, name);
 
     nabi_app_load_base_icons();
+#if defined(HAVE_GTK_STATUS_ICON)
+    // 아이콘을 바꾸려면 현재 state를 알아야 한다.
+    // 그런데 state를 알아오려면 애매하고, 사실 테마가 바뀌는 순간에는
+    // text entry에 focus가 없을 것이므로 기본 상태 값으로 줘도
+    // 별 문제가 없을 것이다.
+    if (nabi_tray != NULL)
+	nabi_tray_update_state(nabi_tray, 0);
+#else
     if (nabi_tray != NULL)
 	nabi_tray_load_icons(nabi_tray, nabi->icon_size);
+#endif
 }
 
 void
@@ -1497,7 +1607,7 @@ nabi_app_set_hangul_keyboard(const char *id)
 	gtk_button_set_label(GTK_BUTTON(nabi->keyboard_button), _(name));
     }
 
-    nabi_app_update_tooltips();
+    nabi_tray_icon_update_tooltips();
 }
 
 void
@@ -1508,19 +1618,22 @@ nabi_app_save_config()
 }
 
 static void
-nabi_app_update_tooltips()
+nabi_tray_icon_update_tooltips()
 {
-    if (nabi->tooltips != NULL) {
+    if (nabi_tray != NULL && nabi_tray->icon != NULL) {
 	const char* keyboard_name;
 	char tip_text[256];
 	keyboard_name = nabi_server_get_keyboard_name_by_id(nabi_server,
 					    nabi->config->hangul_keyboard->str);
 	snprintf(tip_text, sizeof(tip_text), _("Nabi: %s"), _(keyboard_name));
-
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(nabi->tooltips), nabi->tray_icon,
+#if defined(HAVE_GTK_STATUS_ICON)
+	gtk_status_icon_set_tooltip_text(nabi_tray->icon, tip_text);
+#else
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(nabi_tray->tooltips), nabi->tray_icon,
 			     tip_text,
 			     _("Hangul input method: Nabi"
 			       " - You can input hangul using this program"));
+#endif
     }
 }
 
