@@ -43,11 +43,18 @@ public:
     static void preeditDrawCallback(XIM xim, XPointer client_data, XPointer data);
     static void preeditCaretCallback(XIM xim, XPointer client_data, XPointer data);
 
+    static void statusStartCallback(XIM xim, XPointer client_data, XPointer data);
+    static void statusDoneCallback(XIM xim, XPointer client_data, XPointer data);
+    static void statusDrawCallback(XIM xim, XPointer client_data, XPointer data);
+    
+    // string conversion callback
+    static void stringConversionCallback(XIM xim, XPointer client_data, XPointer data);
+
 private:
     // for XIM interaction
     void openIM();
     void closeIM();
-    void createIC();
+    void createIC(bool useStringConversion);
     void destroyIC();
 
     void setPreeditString(const wchar_t *str, int pos, int length);
@@ -150,7 +157,24 @@ void TextView::openIM()
     destroy.client_data = (XPointer)this;
     XSetIMValues(m_im, XNDestroyCallback, &destroy, NULL);
 
-    createIC();
+    bool useStringConversion = false;
+
+    XIMValuesList* ic_values = NULL;
+    XGetIMValues(m_im,
+		 XNQueryICValuesList, &ic_values,
+		 NULL);
+    
+    if (ic_values != NULL) {
+	for (int i = 0; i < ic_values->count_values; i++) {
+	    if (strcmp(ic_values->supported_values[i],
+		       XNStringConversionCallback) == 0) {
+		useStringConversion = true;
+		break;
+	    }
+	}
+    }
+
+    createIC(useStringConversion);
 }
 
 void TextView::closeIM(void)
@@ -161,7 +185,7 @@ void TextView::closeIM(void)
     printf("XIM is closed\n");
 }
 
-void TextView::createIC(void)
+void TextView::createIC(bool useStringConversion)
 {
     if (m_im == NULL)
 	return;
@@ -179,18 +203,36 @@ void TextView::createIC(void)
 	preedit_draw.client_data = (XPointer)this;
 	preedit_caret.callback = preeditCaretCallback;
 	preedit_caret.client_data = (XPointer)this;
-	XVaNestedList attr = XVaCreateNestedList(0,
+	XVaNestedList p_attr = XVaCreateNestedList(0,
 				 XNPreeditStartCallback, &preedit_start,
 				 XNPreeditDoneCallback,  &preedit_done,
 				 XNPreeditDrawCallback,  &preedit_draw,
 				 XNPreeditCaretCallback, &preedit_caret,
 				 NULL);
+
+	XIMCallback status_start;
+	XIMCallback status_done;
+	XIMCallback status_draw;
+	status_start.callback = statusStartCallback;
+	status_start.client_data = (XPointer)this;
+	status_done.callback = statusDoneCallback;
+	status_done.client_data = (XPointer)this;
+	status_draw.callback = statusDrawCallback;
+	status_draw.client_data = (XPointer)this;
+	XVaNestedList s_attr = XVaCreateNestedList(0,
+				 XNStatusStartCallback, &status_start,
+				 XNStatusDoneCallback,  &status_done,
+				 XNStatusDrawCallback,  &status_draw,
+				 NULL);
+
 	m_ic = XCreateIC(m_im, 
 			 XNInputStyle, XIMPreeditCallbacks,
 			 XNClientWindow, m_window,
-			 XNPreeditAttributes, attr,
+			 XNPreeditAttributes, p_attr,
+			 XNStatusAttributes, s_attr,
 			 NULL);
-	XFree(attr);
+	XFree(p_attr);
+	XFree(s_attr);
     } else if ((m_inputStyle & XIMPreeditPosition) == XIMPreeditPosition) {
 	XRectangle area;
 	area.x = 0;
@@ -246,6 +288,13 @@ void TextView::createIC(void)
 	unsigned long mask = ExposureMask | KeyPressMask | FocusChangeMask;
 	XSelectInput(m_display, m_window, mask | fevent);
 
+	if (useStringConversion) {
+	    XIMCallback strconv;
+	    strconv.callback    = stringConversionCallback;
+	    strconv.client_data = (XPointer)this;
+	    XSetICValues(m_ic, XNStringConversionCallback, &strconv, NULL);
+	}
+
 	printf("XIC is created\n");
     } else {
 	printf("cannot create XIC\n");
@@ -721,6 +770,87 @@ void TextView::preeditCaretCallback(XIM xim, XPointer user_data, XPointer data)
     textview->draw();
 }
 
+void TextView::statusStartCallback(XIM xim, XPointer user_data, XPointer data)
+{
+    printf("status start\n");
+}
+
+void TextView::statusDoneCallback(XIM xim, XPointer user_data, XPointer data)
+{
+    printf("status done\n");
+}
+
+void TextView::statusDrawCallback(XIM xim, XPointer user_data, XPointer data)
+{
+    printf("status draw\n");
+}
+
+void TextView::stringConversionCallback(XIM xim, XPointer client_data, XPointer data)
+{
+    short position;
+    TextView *textview = reinterpret_cast<TextView*>(client_data);
+
+    XIMStringConversionCallbackStruct* strconv;
+    strconv = reinterpret_cast<XIMStringConversionCallbackStruct*>(data);
+
+    position = (short)strconv->position;
+
+    printf("position:  %d\n", position);
+    printf("direction: %d\n", strconv->direction);
+    printf("operation: %d\n", strconv->operation);
+    printf("factor:    %d\n", strconv->factor);
+
+    std::wstring* curline = textview->m_text[textview->m_caret.y];
+
+    ssize_t begin = textview->m_caret.x + position;
+    size_t end = begin;
+
+    if (strconv->direction == XIMBackwardChar) {
+	begin -= strconv->factor;
+    } else if (strconv->direction == XIMForwardChar) {
+	end += strconv->factor;
+    } else if (strconv->direction == XIMBackwardWord) {
+    } else if (strconv->direction == XIMForwardWord) {
+    }
+
+    if (begin < 0)
+	begin = 0;
+    if ((size_t)begin > curline->length())
+	begin = curline->length();
+
+    if (end > curline->length())
+	end = curline->length();
+
+    if (strconv->operation == XIMStringConversionRetrieval) {
+	std::wstring ret;
+	if (end > (size_t)begin)
+	    ret.assign(*curline, begin, end - begin);
+
+	if (!ret.empty()) {
+	    XIMStringConversionText* text;
+	    text = (XIMStringConversionText*)malloc(sizeof(*text));
+	    if (text != NULL) {
+		size_t size_in_bytes;
+		text->length = ret.length();
+		text->feedback = NULL;
+		text->encoding_is_wchar = True;
+		size_in_bytes = sizeof(wchar_t) * (text->length + 1);
+		text->string.wcs = (wchar_t*)malloc(size_in_bytes);
+		if (text->string.wcs != NULL) {
+		    memcpy(text->string.wcs, ret.c_str(), size_in_bytes);
+		    strconv->text = text;
+		} else {
+		    free(text);
+		}
+	    }
+	}
+    } else if (strconv->operation == XIMStringConversionSubstitution) {
+	if (end > (size_t)begin)
+	    curline->erase(begin, end - begin);
+	    textview->m_caret.x = begin;
+    }
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -753,10 +883,10 @@ main(int argc, char *argv[])
     printf("modifiers: %s\n", modifiers);
 
     int inputStyle = XIMPreeditCallbacks;
-    char *title = "XIM client - On the spot";
+    const char *title = "XIM client - On the spot";
     if (argc >= 2) {
 	if (strcmp(argv[1], "-on") == 0) {
-	    inputStyle = XIMPreeditCallbacks;
+	    inputStyle = XIMPreeditCallbacks | XIMStatusCallbacks;
 	    title = "XIM client - On the spot";
 	} else if (strcmp(argv[1], "-over") == 0) {
 	    inputStyle = XIMPreeditPosition;
